package be.iminds.ilabt.jfed.util;

import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import javafx.scene.control.ButtonType;
import javafx.stage.Window;
import org.controlsfx.dialog.ExceptionDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * JFedCommonDialogs
 */
public class JFedCommonDialogs {
    private static final Logger LOG = LoggerFactory.getLogger(JFedCommonDialogs.class);

    public static void requestEraseJFedConfiguration(Window ownerWindow) {


        JFDialogs.create()
                .message("Resetting jFed clears all settings and slice recovery information.\n" +
                        "Do you want to proceed?")
                .title("WARNING: Data will be lost!")
                .masthead("WARNING: Data will be lost!")
                .owner(ownerWindow)
                .showConfirm()
                .ifPresent(response -> {
                    if (response == ButtonType.YES) {

                        JFDialogs.create()
                                .message("jFed will exit after resetting. Please start jFed again to continue.")
                                .owner(ownerWindow)
                                .title("WARNING: Data will be lost!")
                                .masthead("WARNING: Data will be lost!").buttonTypes(ButtonType.OK, ButtonType.CANCEL)
                                .showWarning().ifPresent(response2 -> {
                            if (response2 == ButtonType.OK) {

                                //erase the preferences directory
                                File jFedDir = JFedUtils.getUserDataDirectoryFile();
                                if (jFedDir.exists() && jFedDir.isDirectory()) {
                                    try {
                                        deleteFullDirectory(jFedDir);
                                    } catch (IOException e) {
                                        LOG.warn("Error while deleting jFed dir " + jFedDir.getAbsolutePath() + " " + e.getMessage(), e);
                                        ExceptionDialog dlg = new ExceptionDialog(e);
                                        dlg.initOwner(ownerWindow);
                                        dlg.setContentText("Error while deleting jFed dir: " + e.getMessage());
                                        dlg.setTitle("Error deleting jFed directory");
                                        dlg.setHeaderText("Error deleting jFed directory");

                                        dlg.showAndWait();
                                    }
                                }

                                Platform.exit();
                                System.exit(0);
                            }
                        });
                    }
                });
    }

    public static void deleteFullDirectory(File f) throws IOException {
        if (f.isDirectory()) {
            for (File c : f.listFiles())
                deleteFullDirectory(c);
        }
        if (!f.delete())
            throw new FileNotFoundException("Failed to delete file: " + f);
    }
}
