/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package be.iminds.ilabt.jfed.experimenter_gui.editor.ontology.view;

import be.iminds.ilabt.jfed.experimenter_gui.editor.ontology.webmap.DrawingWebMap;
import be.iminds.ilabt.jfed.util.ExecuteOnNotNull;
import javafx.beans.InvalidationListener;
import javafx.beans.binding.StringBinding;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.geometry.Insets;
import javafx.geometry.Point2D;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.StringConverter;

import java.text.DecimalFormat;
import java.text.ParseException;

/**
 * @author Patrick
 */
public abstract class MapFilterView extends FilterCriteriaView {
    private static final DecimalFormat FOUR_DIGIT_DECIMALFORMAT = new DecimalFormat("#.####");
    private static final StringConverter<Double> FOUR_DIGIT_DOUBLE_STRINGCONVERTER = new StringConverter<Double>() {


        @Override public String toString(Double value) {
            // If the specified value is null, return a zero-length String
            if (value == null) {
                return "";
            }

            return FOUR_DIGIT_DECIMALFORMAT.format(value);
        }

        @Override public Double fromString(String value) {
            try {
                // If the specified value is null or zero-length, return null
                if (value == null) {
                    return null;
                }

                value = value.trim();

                if (value.length() < 1) {
                    return null;
                }

                // Perform the requested parsing
                return FOUR_DIGIT_DECIMALFORMAT.parse(value).doubleValue();
            } catch (ParseException ex) {
                throw new RuntimeException(ex);
            }
        }
    };


    protected final ReadOnlyObjectProperty<Double> minLat, maxLat;
    protected final ReadOnlyObjectProperty<Double> minLon, maxLon;
    private final String title;
    private final SpinnerValueFactory.DoubleSpinnerValueFactory minLatValueFactory;
    private final SpinnerValueFactory.DoubleSpinnerValueFactory maxLatValueFactory;
    private final SpinnerValueFactory.DoubleSpinnerValueFactory minLonValueFactory;
    private final SpinnerValueFactory.DoubleSpinnerValueFactory maxLonValueFactory;

    //popup fileds
    private Stage stage;
    private Spinner<Double> minLatField, maxLatField;
    private Spinner<Double> minLonField, maxLonField;

    private final DrawingWebMap drawingWebMap;

    private Button okButton;

    private boolean updatingRectangle = false;

    public MapFilterView() {
        this.title = "Location";

        //initialize Popup
        stage = new Stage(StageStyle.TRANSPARENT);
        StackPane layout = new StackPane();
        VBox popupVBox = new VBox(5);
        minLatValueFactory = new SpinnerValueFactory.DoubleSpinnerValueFactory(-90, 90, -45, 1);
        minLatValueFactory.setConverter(FOUR_DIGIT_DOUBLE_STRINGCONVERTER);
        minLatField = new Spinner<>(minLatValueFactory);
        minLatField.setEditable(true);
        minLat = minLatField.valueProperty();

        maxLatValueFactory = new SpinnerValueFactory.DoubleSpinnerValueFactory(-90, 90, 45, 1);
        maxLatValueFactory.setConverter(FOUR_DIGIT_DOUBLE_STRINGCONVERTER);
        maxLatField = new Spinner<>(maxLatValueFactory);
        maxLatField.setEditable(true);
        maxLat = maxLatField.valueProperty();

        minLonValueFactory = new SpinnerValueFactory.DoubleSpinnerValueFactory(-180, 180, -90, 1);
        minLonValueFactory.setConverter(FOUR_DIGIT_DOUBLE_STRINGCONVERTER);
        minLonField = new Spinner<>(minLonValueFactory);
        minLonField.setEditable(true);
        minLon = minLonField.valueProperty();

        maxLonValueFactory = new SpinnerValueFactory.DoubleSpinnerValueFactory(-180, 180, 90, 1);
        maxLonValueFactory.setConverter(FOUR_DIGIT_DOUBLE_STRINGCONVERTER);
        maxLonField = new Spinner<>(maxLonValueFactory);
        maxLonField.setEditable(true);
        maxLon = maxLonField.valueProperty();


        okButton = new Button("OK");
        okButton.setOnAction(event -> {
            hideStage();
        });

        HBox hBoxLat = new HBox();
        hBoxLat.getChildren().addAll(new Label("Latitude: "), new Label("Min: "), minLatField, new Label("Max: "), maxLatField);
        HBox hBoxLon = new HBox();
        hBoxLon.getChildren().addAll(new Label("Longitude: "), new Label("Min: "), minLonField, new Label("Max: "), maxLonField);

        drawingWebMap = new DrawingWebMap();
        drawingWebMap.setPrefHeight(400);
        drawingWebMap.setPrefWidth(600);


        HBox hBox2 = new HBox();
        hBox2.setAlignment(Pos.CENTER_RIGHT);
        hBox2.getChildren().addAll(okButton);

        popupVBox.setPadding(new Insets(5));
        popupVBox.getChildren().addAll(hBoxLat, hBoxLon, drawingWebMap, hBox2);
        popupVBox.setStyle("-fx-border-color: grey;"
                + "-fx-border-width: 2px;");
        layout.getChildren().addAll(popupVBox);

        Scene scene = new Scene(layout, Color.LIGHTGREY);
        stage.setScene(scene);
        stage.setResizable(false);

        this.setOnAction(event -> {
            ToggleButton button = (ToggleButton) event.getSource();
            if (stage.isShowing()) {
                stage.hide();
            } else {
                Scene scene1 = button.getScene();
                final Point2D windowCoord = new Point2D(scene1.getWindow().getX(), scene1.getWindow().getY());
                final Point2D sceneCoord = new Point2D(scene1.getX(), scene1.getY());
                final Point2D nodeCoord = button.localToScene(0.0, 0.0);
                stage.setX(windowCoord.getX() + sceneCoord.getX() + nodeCoord.getX());
                stage.setY(windowCoord.getY() + sceneCoord.getY() + nodeCoord.getY() + button.getHeight());
                stage.show();
                okButton.requestFocus();
            }
        });

        stage.setOnHidden(event -> setSelected(false));

        //bind text to underlying values
        textProperty().bind(new StringBinding() {
            {
                bind(minLat);
                bind(maxLat);
                bind(minLon);
                bind(maxLon);
            }

            @Override
            protected String computeValue() {
                String result = title + " ";

                boolean hasLat = true;
                if (minLat.get() > -180 && maxLat.get() < 180) {
                    result += "lat: [" + FOUR_DIGIT_DECIMALFORMAT.format(minLat.get()) + ", " + FOUR_DIGIT_DECIMALFORMAT.format(maxLat.get()) + "]°";
                } else if (minLat.get() > -180) {
                    result += "lat: >= " + FOUR_DIGIT_DECIMALFORMAT.format(minLat.get()) + "°";

                } else if (maxLat.get() < 180) {
                    result += "lat: <= " + FOUR_DIGIT_DECIMALFORMAT.format(maxLat.get()) + "°";
                } else {
                    hasLat = false;
                }

                if (hasLat)
                    result += " ";

                if (minLon.get() > -180 && maxLon.get() < 180) {
                    result += "lon: [" + FOUR_DIGIT_DECIMALFORMAT.format(minLon.get()) + ", " + FOUR_DIGIT_DECIMALFORMAT.format(maxLon.get()) + "]°";
                } else if (minLon.get() > -180) {
                    result += "lon: >= " + FOUR_DIGIT_DECIMALFORMAT.format(minLon.get()) + "°";

                } else if (maxLon.get() < 180) {
                    result += "lon: <= " + FOUR_DIGIT_DECIMALFORMAT.format(maxLon.get()) + "°";
                }

                return result;
            }
        });

        new ExecuteOnNotNull<Scene>(sceneProperty()) {
            @Override
            public void run() {
                stage.initOwner(getScene().getWindow());
            }
        };

        final InvalidationListener webMapListener = observable -> {
            if (!updatingRectangle)
                drawingWebMap.setRectangle(new DrawingWebMap.MapRectangle(minLat.get(), maxLat.get(), minLon.get(), maxLon.get()));
        };

        minLat.addListener(webMapListener);
        maxLat.addListener(webMapListener);
        minLon.addListener(webMapListener);
        maxLon.addListener(webMapListener);

        webMapListener.invalidated(null); //force first update

        drawingWebMap.rectangleProperty().addListener(observable -> {
            updatingRectangle = true;
            if (drawingWebMap.getRectangle() != null) {
                minLatField.getValueFactory().setValue(drawingWebMap.getRectangle().getMinLat());
                maxLatField.getValueFactory().setValue(drawingWebMap.getRectangle().getMaxLat());
                minLonField.getValueFactory().setValue(drawingWebMap.getRectangle().getMinLon());
                maxLonField.getValueFactory().setValue(drawingWebMap.getRectangle().getMaxLon());
            }

            updatingRectangle = false;
        });

        //allow 4 digits in spinners


    }

    public void hideStage() {
        stage.hide();
    }

    public String getTitle() {
        return title;
    }

    @Override
    public String getName() {
        return title;
    }

    @Override
    public boolean isValid() {
        return minLat.get() < maxLat.get() && minLon.get() < maxLon.get();
    }

    public Double getMinLat() {
        return minLat.get();
    }

    public ReadOnlyObjectProperty<Double> minLatProperty() {
        return minLat;
    }

    public Double getMaxLat() {
        return maxLat.get();
    }

    public ReadOnlyObjectProperty<Double> maxLatProperty() {
        return maxLat;
    }

    public Double getMinLon() {
        return minLon.get();
    }

    public ReadOnlyObjectProperty<Double> minLonProperty() {
        return minLon;
    }

    public Double getMaxLon() {
        return maxLon.get();
    }

    public ReadOnlyObjectProperty<Double> maxLonProperty() {
        return maxLon;
    }
}
