package be.iminds.ilabt.jfed;

import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.SimpleGeniUser;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Named;

/**
 * BaseCliModule
 */
public class DirectUserModule extends AbstractModule {
    private final static Logger LOG = LoggerFactory.getLogger(DirectUserModule.class);

    private final SimpleGeniUser geniUser;

    public DirectUserModule(SimpleGeniUser geniUser) {
        this.geniUser = geniUser;
    }

    @Override
    protected void configure() {
        //we use provides, so we can fill in the user authority based on the AuthorityListModel

//        assert geniUser != null;
//        bind(GeniUser.class).toInstance(geniUser);

//        assert geniUser.getUserAuthority() != null;
//        bind(SfaAuthority.class).annotatedWith(Names.named("userAuthority")).toInstance(geniUser.getUserAuthority());
    }

    @Provides
    public GeniUser provideGeniUser(AuthorityListModel authListModel, AuthorityFinder authFinder) {
            if (geniUser.getUserAuthority() == null) {
                geniUser.deriveUserAndAuthFromPemContent(authFinder);
                if (geniUser.getUserAuthority() == null)
                    throw new RuntimeException("The PEM file with user information did not contain a known Authority. " +
                            "Causes: it either did not contain this information in the certificate, " +
                            "or the authority is not known by jFed (possible due to configuration error)");
            }

            return geniUser;
        }

    @Provides
    @Named("userAuthority")
    public SfaAuthority provideUserAuthority(AuthorityListModel authListModel, AuthorityFinder authFinder) {
        if (geniUser.getUserAuthority() == null) {
            geniUser.deriveUserAndAuthFromPemContent(authFinder);
            if (geniUser.getUserAuthority() == null)
                throw new RuntimeException("The PEM file with user information did not contain a known Authority. " +
                        "Causes: it either did not contain this information in the certificate, " +
                        "or the authority is not known by jFed (possible due to configuration error)");
        }

        return geniUser.getUserAuthority();
    }

    @Provides
    public GeniUserProvider provideGeniUserProvider(AuthorityListModel authListModel, AuthorityFinder authFinder) {
        if (geniUser.getUserAuthority() == null) {
            geniUser.deriveUserAndAuthFromPemContent(authFinder);
            if (geniUser.getUserAuthority() == null)
                throw new RuntimeException("The PEM file with user information did not contain a known Authority. " +
                        "Causes: it either did not contain this information in the certificate, " +
                        "or the authority is not known by jFed (possible due to configuration error)");
        }

        return new GeniUserProvider() {
            @Override
            public GeniUser getLoggedInGeniUser() {
                return geniUser;
            }

            @Override
            public boolean isUserLoggedIn() {
                return geniUser != null;
            }
        };
    }
}
