package be.iminds.ilabt.jfed.ui.javafx.scanner;

import be.iminds.ilabt.jfed.BaseGuiModule;
import be.iminds.ilabt.jfed.UncaughtGuiExceptionLogger;
import be.iminds.ilabt.jfed.highlevel.LowLevelModule;
import be.iminds.ilabt.jfed.highlevel.UserLoginModelModule;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.ui.javafx.userlogin.UserLoginFactory;
import be.iminds.ilabt.jfed.ui.javafx.util.ExtraInfoCallBackGUIs;
import com.cathive.fx.guice.GuiceApplication;
import com.google.inject.Module;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.util.List;

public class ScannerStandalone extends GuiceApplication {
    private static final Logger LOG = LoggerFactory.getLogger(ScannerStandalone.class);

    /* solution for MAC OS HeadlessException bug. See https://ibcn-jira.intec.ugent.be/browse/FEDIBBTDEV-239 */
    static {
        System.setProperty("java.awt.headless", "false");
    }

    @Inject
    private UserLoginFactory userLoginFactory;

    @Inject
    private ScanAuthorityDialogFactory scanAuthorityDialogFactory;
    @Inject
    private ScanResultPanelFactory scanResultPanelFactory;


    @Override
    public void start(final Stage stage) throws Exception {
        Thread.setDefaultUncaughtExceptionHandler(UncaughtGuiExceptionLogger.getInstance(stage));

        ExtraInfoCallBackGUIs.prepareExtraInfoCallback(stage);

        stage.setOnCloseRequest(event -> exit());

        userLoginFactory.showUserLogin(false, stage, successfulLogin -> { while (showScanner()) { /*nothing in loop*/ }; exit(); });
    }

    /**
     * The main() method is ignored in correctly deployed JavaFX application.
     * main() serves only as fallback in case the application can not be
     * launched through deployment artifacts, e.g., in IDEs with limited FX
     * support. NetBeans ignores main().
     *
     * @param args the command line arguments
     */
    public static boolean debugEnabled = false;

    public static void main(String[] args) {
        for (String arg : args) {
            if (arg.equalsIgnoreCase("debug") || arg.equalsIgnoreCase("--debug") || arg.equalsIgnoreCase("-debug"))
                debugEnabled = true;
        }
        Application.launch(args);
    }

    @Override
    public void init(List<Module> modules) throws Exception {
        modules.add(new LowLevelModule());
        modules.add(new UserLoginModelModule());
        modules.add(new BaseGuiModule());
    }

    @Inject
    private AuthorityListModel authorityListModel;

    /** returns true if scan again has been chosen */
    public boolean showScanner() {
        SfaAuthority auth = scanAuthorityDialogFactory.showScanAuthorityDialog(ScanAuthorityDialogController.UserChoose.SHOW_AUTH, false/*dialog*/);

//        SfaAuthority auth = authorityListModel.getByUrnExact(GeniUrn.createGeniUrnFromDecodedParts("wall2.ilabt.iminds.be", "authority", "cm")); //for debugging ScanResultPanel

        if (auth == null)
            return false;

        LOG.debug("Showing scanner for auth "+(auth == null ? null : auth.getUrn()));
        boolean scanAgain = scanResultPanelFactory.showScanResultPanel(auth, false/*dialog*/);
        return scanAgain;
    }

    public void exit() {
        LOG.info("Close was requested. Trying to terminate");

        Platform.exit();
        System.exit(0);
    }
}
